#!/usr/bin/env python3

import os
import sys
import pwd
import grp
import stat
import glob
import lib_path
import lib
import posix1e
import dphysldap
import pyxymon as pymon

CHECK_NAME = 'permissions'
CHECK_VERSION = 2
LIFETIME = 30

home_dir = '/export/home1/*'
owner = []
permission = []
acl = []
home_dirs = 0
users = {}
nis_homes = {}
nis_shares = {}
bad_home_directory = []
no_ldap_user = []
bad_nis_map = []
no_nis_map = []


class Home(object):
    """
    Holds info about a home directory
    """
    def __init__(self, name, path, st):
        self.name = name
        self.path = path
        self.st = st

    @property
    def uid(self):
        return self.st.st_uid

    @property
    def gid(self):
        return self.st.st_gid

    @property
    def uname(self):
        return pwd.getpwuid(self.uid).pw_name

    @property
    def gname(self):
        return grp.getgrgid(self.gid).gr_name

    @property
    def filemode(self):
        return stat.filemode(self.st.st_mode)

    @property
    def permission(self):
        return ' '.join([self.filemode, self.path])

    def __str__(self):
        return ' '.join([self.filemode, self.uname, self.gname, self.path])


def search_ldap():
    ldap = dphysldap.Ldap()
    ldap_users = dphysldap.Users(ldap, ['uid', 'uidNumber', 'gidNumber', 'homeDirectory'])
    entries = dphysldap.Entries(ldap, ['cn', 'nisMapEntry'])
    auto_home = 'nisMapName=auto.home,ou=automount,dc=phys,dc=ethz,dc=ch'

    ldap_users.search('*')
    for user in ldap_users:
        users[user['uid'][0]] = user['homeDirectory'][0]

    entries.search('cn: *, nisMapEntry: phd-home*', ['nisObject'], base=auto_home)
    for entry in entries:
        nis_homes[entry['cn'][0]] = entry['nisMapEntry'][0]

    entries.search('cn: *, nisMapEntry: != phd-home*', ['nisObject'], base=auto_home)
    for entry in entries:
        nis_shares[entry['cn'][0]] = entry['nisMapEntry'][0]


def check_homes(top):
    global home_dirs

    if not os.path.isdir(top):
        return
    for f in os.listdir(top):
        pathname = os.path.join(top, f)
        if not os.path.isdir(pathname):
            continue
        st = os.stat(pathname)
        home = Home(f, pathname, st)
        home_dirs += 1

        if bad_owner(home):
            owner.append(home)

        if bad_permission(home):
            permission.append(home)

        if posix1e.has_extended(home.path):
            acl.append(home)

        if home.name in users:
            if users[home.name][6:] != home.name:
                bad_home_directory.append(': '.join([home.name, users[home.name]]))
            del users[home.name]
        else:
            no_ldap_user.append(home)

        if home.name in nis_homes:
            if nis_homes[home.name].split(':', maxsplit=1)[1] != home.path:
                bad_nis_map.append(': '.join([home.name, nis_homes[home.name]]))
            del nis_homes[home.name]
        else:
            no_nis_map.append(home)


def check_shares():
    for user in list(users.keys()):
        if user in nis_shares:
            del nis_shares[user]
            del users[user]


def bad_owner(home):
    if home.name == home.uname and home.name == home.gname:
        return False
    return True


def bad_permission(home):
    # d---------
    if home.st.st_mode == stat.S_IFDIR:
        return False
    # drwx------
    elif home.st.st_mode == stat.S_IFDIR | stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR:
        return False
    return True


def list_homes(homes):
    for home in homes:
        print(home)


def run_check(xymon):
    search_ldap()

    title = 'statistics'
    content = ''
    content += 'ldap users:           ' + str(len(users)) + '<br/>'
    content += 'ldap nismaps (home):  ' + str(len(nis_homes)) + '<br/>'
    content += 'ldap nismaps (share): ' + str(len(nis_shares)) + '<br/>'

    for path in glob.glob(home_dir):
        check_homes(path)

    check_shares()

    content += 'home dirs:            ' + str(home_dirs) + '<br/>'
    content += 'strange ldap users:   ' + str(len(users)) + '<br/>'
    content += 'orphaned nis homes:   ' + str(len(nis_homes)) + '<br/>'
    content += 'orphaned nis shares:  ' + str(len(nis_shares)) + '<br/>'
    content += 'bad homeDirectory:    ' + str(len(bad_home_directory)) + '<br/>'
    content += 'no user for home:     ' + str(len(no_ldap_user)) + '<br/>'
    content += 'bad nismaps (home):   ' + str(len(bad_nis_map)) + '<br/>'
    content += 'no nismap for home:   ' + str(len(no_nis_map)) + '<br/>'

    xymon.section(title, content)

    if owner:
        title = 'bad owner or group'
        content = 'home must be owned by the respective user and the group his user-private-group<br/><br/>'
        for home in owner:
            content += ''.join([str(home), '<br/>'])
        xymon.section(title, content)
        xymon.color = pymon.STATUS_CRITICAL

    if permission:
        title = 'bad permissions'
        content = 'home permission not <code>drwx------</code> (active user) or <code>d---------</code> (blocked user)<br/><br/>'
        for home in permission:
            content += ''.join([str(home.permission), '<br/>'])
        xymon.section(title, content)
        xymon.color = pymon.STATUS_CRITICAL

    if acl:
        title = 'bad acls'
        content = 'home has posix.1e extended ACLs<br/>check acls using `getfacl`, which stands for `get fucking ACL`<br/><br/>'
        for home in acl:
            extacl = posix1e.ACL(file=home.path)
            content += ''.join([home.path, '<br/>'])
            content += ''.join([str(extacl), '<br/>'])
        xymon.section(title, content)
        xymon.color = pymon.STATUS_CRITICAL

    if users:
        title = 'strange ldap users'
        content = 'these users seem to not have a home directory on the filesystem<br/><br/>'
        for k, v in users.items():
            content += ': '.join([k, v]) + '<br/>'
        xymon.section(title, content)
        xymon.color = pymon.STATUS_CRITICAL

    if nis_homes:
        title = 'orphaned nis homes'
        content = 'these nismaps do not have a home directory on the filesystem<br/><br/>'
        for k, v in nis_homes.items():
            content += ': '.join([k, v]) + '<br/>'
        xymon.section(title, content)
        xymon.color = pymon.STATUS_CRITICAL

    if nis_shares:
        title = 'orphaned nis shares'
        content = 'these nismaps do not have a user in ldap<br/><br/>'
        for k, v in nis_shares.items():
            content += ': '.join([k, v]) + '<br/>'
        xymon.section(title, content)
        xymon.color = pymon.STATUS_CRITICAL

    if bad_home_directory:
        title = 'bad homeDirectory'
        content = 'the homeDirectory attributes home name does not match the username<br/><br/>'
        for home in bad_home_directory:
            content += ''.join([str(home), '<br/>'])
        xymon.section(title, content)
        xymon.color = pymon.STATUS_CRITICAL

    if no_ldap_user:
        title = 'no user for home'
        content = 'home directory without a corresponding ldap user<br/><br/>'
        for home in no_ldap_user:
            content += ''.join([str(home), '<br/>'])
        xymon.section(title, content)
        xymon.color = pymon.STATUS_CRITICAL

    if bad_nis_map:
        title = 'bad nismaps (home)'
        content = 'nismap does not match home path on the filesystem<br/><br/>'
        for home in bad_nis_map:
            content += ''.join([str(home), '<br/>'])
        xymon.section(title, content)
        xymon.color = pymon.STATUS_CRITICAL

    if no_nis_map:
        title = 'no nismap for home'
        content = 'home directory without a corresponding nismap entry<br/><br/>'
        for home in no_nis_map:
            content += ''.join([str(home), '<br/>'])
        xymon.section(title, content)
        xymon.color = pymon.STATUS_CRITICAL


def main():
    """Run xymon check"""
    xymon = pymon.XymonClient(CHECK_NAME)
    check_script = os.path.basename(__file__)
    # The default criticity is set to 'pymon.STATUS_OK'
    xymon.lifetime = LIFETIME
    xymon.title('home ownership and permissions')

    try:
        run_check(xymon)
    except Exception as e:
        xymon.color = pymon.STATUS_WARNING
        xymon.section('Exception', e)

    xymon.footer(check_script, CHECK_VERSION)
    xymon.send()


if __name__ == '__main__':
    main()
    sys.exit(0)
